const { readFileSync, writeFileSync } = require("fs");
const { join } = require("path");
const { ios, android } = require("@raydeck/react-native-utilities");
const { parseStringPromise, Builder } = require("xml2js");
const isAndroid = true;
const isIos = true;
const path = process.cwd();
module.exports = {
  commands: [
    {
      name: "add-scheme <scheme>",
      description:
        "Add URL Scheme for your app to auto-open (note this should be app-specific and not https, etc)",
      func: ([scheme]) => {
        const packagePath = join(process.cwd(), "package.json");
        const o = JSON.parse(readFileSync(packagePath, { encoding: "utf8" }));
        if (!o.urlSchemes) o.urlSchemes = [];
        if (o.urlSchemes.some((name) => scheme === name)) {
          console.warn(
            "URL Scheme is already associated with this app",
            scheme
          );
          return;
        }
        o.urlSchemes.push(scheme);
        writeFileSync(packagePath, JSON.stringify(o, null, 2));
      },
    },
    {
      name: "list-schemes",
      description: "List all URL schemes for this app",
      func: () => {
        const packagePath = join(process.cwd(), "package.json");
        const o = JSON.parse(readFileSync(packagePath, { encoding: "utf8" }));
        if (!o.urlSchemes || !o.urlSchemes.length)
          console.warn("No Schemes saved on this project");
        else {
          console.log("URL Schemes saved on this app:");
          o.urlSchemes.map((scheme) => {
            console.log("\t" + scheme);
          });
        }
      },
    },
    {
      name: "remove-scheme <scheme>",
      description: "Remove url scheme for this app",
      func: ([scheme]) => {
        const packagePath = join(process.cwd(), "package.json");
        const o = JSON.parse(readFileSync(packagePath, { encoding: "utf8" }));
        if (!o.urlSchemes || !o.urlSchemes.length)
          console.warn("No Schemes saved on this project");
        else {
          o.urlSchemes = o.urlSchemes.filter((s) => s !== scheme);
          if (!o.urlSchemes.length) delete o.urlSchemes;
          writeFileSync(packagePath, JSON.stringify(o, null, 2));
          console.log("Done removing ", scheme);
        }
      },
    },
    {
      name: "clear-schemes",
      description: "Remove all url scheme for this app",
      func: () => {
        const packagePath = join(process.cwd(), "package.json");
        const o = JSON.parse(readFileSync(packagePath, { encoding: "utf8" }));
        if (!o.urlSchemes) console.warn("No Schemes saved on this project");
        else {
          delete o.urlSchemes;
          writeFileSync(packagePath, JSON.stringify(o, null, 2));
        }
      },
    },
    {
      name: "add-domain <domain>",
      description:
        "Add a web domain (requires applinks and apple-app-association files on the site",
      func: async ([domain]) => {
        if (isIos) {
          const domains = (
            ios.getEntitlement("com.apple.developer.associated-domains") || []
          ).filter((d) => d !== "applinks:" + domain);
          domains.push("applinks:" + domain);
          ios.setEntitlement("com.apple.developer.associated-domains", domains);
        }
        if (isAndroid) {
          const xmlPath = join(
            android.getMainPath(path),
            "AndroidManifest.xml"
          );
          const xml = readFileSync(xmlPath, { encoding: "utf8" });
          const o = await parseStringPromise(xml);
          //Look for the main activity
          //Find a launchmode of singletask
          const activity = o.manifest.application[0].activity.find(
            ({ $: { "android:name": name } }) => name === ".MainActivity"
          );
          if (!activity) throw "No Main activity specified!";
          if (!activity["intent-filter"]) activity["intent-filter"] = [];
          //Clear all the intent filters related to URI entry
          activity["intent-filter"] = activity["intent-filter"].filter(
            ({ data }) =>
              !data ||
              !data.some(
                ({
                  $: { ["android:scheme"]: scheme, ["android:host"]: host },
                }) => scheme === "https" && host === domain
              )
          );
          activity["intent-filter"].push({
            $: { "android:label": domain },
            action: [{ $: { "android:name": "android.intent.action.VIEW" } }],
            category: [
              { $: { "android:name": "android.intent.category.DEFAULT" } },
              { $: { "android:name": "android.intent.category.BROWSABLE" } },
            ],
            data: [
              { $: { "android:scheme": "https", "android:host": domain } },
            ],
          });
          const out = new Builder().buildObject(o);
          writeFileSync(xmlPath, out);
        }
      },
    },
    {
      name: "remove-domain <domain>",
      description: "Remove domain from app",
      func: async ([domain]) => {
        if (isIos) {
          const domains = ios.getEntitlement(
            "com.apple.developer.associated-domains"
          );
          newdomains = domains.filter((d) => !d.includes(domain));
          ios.setEntitlement(
            "com.apple.developer.associated-domains",
            newdomains
          );
        }
        if (isAndroid) {
          const xmlPath = join(
            android.getMainPath(path),
            "AndroidManifest.xml"
          );
          const xml = readFileSync(xmlPath, { encoding: "utf8" });
          const o = await parseStringPromise(xml);
          const activity = o.manifest.application[0].activity.find(
            ({ $: { "android:name": name } }) => name === ".MainActivity"
          );
          if (!activity) throw "No Main activity specified!";
          if (!activity["intent-filter"]) activity["intent-filter"] = [];
          //Clear all the intent filters related to URI entry
          activity["intent-filter"] = activity["intent-filter"].filter(
            ({ data }) =>
              !data ||
              !data.some(
                ({
                  $: { ["android:scheme"]: scheme, ["android:host"]: host },
                }) => scheme === "https" && host.includes(domain)
              )
          );
          const out = new Builder().buildObject(o);
          writeFileSync(xmlPath, out);
        }
      },
    },
    {
      name: "clear-domains",
      description: "remove all associated domains from app",
      func: async () => {
        if (isIos) {
          ios.setEntitlement("com.apple.developer.associated-domains", []);
        }
        if (isAndroid) {
          const xmlPath = join(
            android.getMainPath(path),
            "AndroidManifest.xml"
          );
          const xml = readFileSync(xmlPath, { encoding: "utf8" });
          const o = await parseStringPromise(xml);
          const activity = o.manifest.application[0].activity.find(
            ({ $: { "android:name": name } }) => name === ".MainActivity"
          );
          if (!activity) throw "No Main activity specified!";
          if (!activity["intent-filter"]) activity["intent-filter"] = [];
          //Clear all the intent filters related to URI entry
          activity["intent-filter"] = activity["intent-filter"].filter(
            ({ data }) =>
              !data ||
              !data.some(
                ({
                  $: { ["android:scheme"]: scheme, ["android:host"]: host },
                }) => scheme === "https"
              )
          );
          const out = new Builder().buildObject(o);
          writeFileSync(xmlPath, out);
        }
      },
    },
    {
      name: "list-domains",
      description: "List applink domains",
      func: () => {
        const domains = ios.getEntitlement(
          "com.apple.developer.associated-domains"
        );
        console.log(JSON.stringify(domains, null, 2));
      },
    },
    {
      name: "make-domainfile",
      description:
        "Create a simple apple or android domain link file to go in .well-known",
      options: [
        {
          name: "-i --ios",
          description: "Create apple/iOS version of file",
        },
        { name: "-a --android", description: "Create android version of file" },
        {
          name: "-o --output-path <path>",
          description: "Path in which to create the files",
          default: process.cwd(),
        },
      ],
      func: async (_, __, { ios: isIos, android: isAndroid, outputPath }) => {
        if (!isIos && !isAndroid)
          console.error(
            "Must specify whether to build for ios, android or both"
          );
        if (isIos) {
          //get bundleid and team id
          const teamId = ios.getBuildProperty("DEVELOPMENT_TEAM");
          const bundleId = ios.getBuildProperty("PRODUCT_BUNDLE_IDENTIFIER");
          const appID = teamId + "." + bundleId;
          const ios13detailObjects = [
            {
              appIDs: [appID],
              components: [{ "/": "/**", comment: "fits all urls" }],
            },
          ];
          const ios12detailObjects = [
            {
              appID,
              paths: ios13detailObjects
                .filter(({ appIDs }) => appIDs.includes(appID))
                .map(({ components }) =>
                  components.map(({ "/": slash }) => slash)
                )
                .flat(),
            },
          ];
          const apps = [appID];
          const o = {
            applinks: {
              apps: [],
              details: [...ios13detailObjects, ...ios12detailObjects],
            },
            webcredentials: { apps },
          };
          writeFileSync(
            join(outputPath, "apple-app-site-association"),
            JSON.stringify(o, null, 2)
          );
        }
        if (isAndroid) {
          const fingerprint = ""; //@TODO
          const androidAppId = await android.getManifestPackageName();
          const androidPath = join(outputPath, "applinks.json");
          const appLink = {
            relation: ["delegate_permission/common.handle_all_urls"],
            target: {
              namespace: "android_app",
              package_name: androidAppId,
              sha256_cert_fingerprints: [fingerprint],
            },
          };
          const appLinks = [appLink];
          writeFileSync(androidPath, JSON.stringify(appLinks, null, 2));
        }
      },
    },
  ],
  advanced: {
    startupClasses: ["ReactNativeAdvancedLinking.RNALinking"],
    prelink: async ({ iosOnly, androidOnly, jsOnly, path = process.cwd() }) => {
      const doIos = iosOnly || (!androidOnly && !jsOnly);
      //get schemes for this project
      const { urlSchemes: temp } = require(join(path, "package.json"));
      const schemes = typeof temp === "string" ? [temp] : temp;
      const doAndroid = androidOnly || (!iosOnly && !jsOnly);
      if (doIos) {
        if (schemes && schemes.length) {
          ios.setPlistValue("CFBundleURLTypes", [
            {
              CFBundleTypeRole: "Viewer",
              CFBundleURLName: ios.getProjectName(),
              CFBundleURLSchemes: schemes,
            },
          ]);
        }
      }
      if (doAndroid) {
        const xmlPath = join(android.getMainPath(path), "AndroidManifest.xml");
        const xml = readFileSync(xmlPath, { encoding: "utf8" });
        const o = await parseStringPromise(xml);
        //Look for the main activity
        //Find a launchmode of singletask
        const activity = o.manifest.application[0].activity.find(
          ({ $: { "android:name": name } }) => name === ".MainActivity"
        );
        // const activity = o.manifest.application[0].activity.find(
        //   ({ "intent-filter": name }) => name
        // );
        console.log("I found an activity", activity);
        if (!activity) throw "No Main activity specified!";
        if (!activity["intent-filter"]) activity["intent-filter"] = [];
        //Clear all the intent filters related to URI entry
        activity["intent-filter"] = activity["intent-filter"].filter(
          ({ data }) =>
            !data ||
            !data.some(({ $: { ["android:scheme"]: scheme } }) => scheme)
        );
        if (schemes && schemes.length)
          schemes.forEach((scheme) => {
            activity["intent-filter"].push({
              $: { "android:label": scheme },
              action: [{ $: { "android:name": "android.intent.action.VIEW" } }],
              category: [
                { $: { "android:name": "android.intent.category.DEFAULT" } },
                { $: { "android:name": "android.intent.category.BROWSABLE" } },
              ],
              data: [{ $: { "android:scheme": scheme } }],
            });
          });
        const out = new Builder().buildObject(o);
        writeFileSync(xmlPath, out);
      }
    },
  },
};
